package ij.measure;

/** Calibration objects contain an image's spatial and density calibration data. */
   
public class Calibration {

	public static final int STRAIGHT_LINE=0,POLY2=1,POLY3=2,POLY4=3,
		EXPONENTIAL=4,POWER=5,LOG=6,RODBARD=7;
	public static final int NONE=20,INVERTER=21,UNCALIBRATED_OD=22;

	/** Pixel width in 'unit's */
	public double pixelWidth = 1.0;
	
	/** Pixel height in 'unit's */
	public double pixelHeight = 1.0;
	
	/** Pixel depth in 'unit's */
	public double pixelDepth = 1.0;

	/** Calibration function coefficients */
	public double[] coefficients;
		
	/* Distance unit (e.g. 'cm', 'inch') */
	private String unit = "pixel";
	
	/* Distance units (e.g. 'microns', 'inches') */
	private String units;

	/* Pixel value unit (e.g. 'gray level', 'OD') */
	private String valueUnit = "Gray Value";

	/* Calibration function ID */
	private int function = NONE;

	/* Calibration table */
	private float[] cTable;

	/** Constructs a new Calibration object using the default values. */ 
	public Calibration() {
	}
	
	/** Returns true if this image is spatially calibrated. */
	public boolean scaled() {
		return pixelWidth!=1.0 || pixelHeight!=1.0;
	}
	
   	/** Sets the distance unit (e.g. "mm", "inch"). */
 	public void setUnit(String unit) {
 		if (unit==null)
 			this.unit = "pixel";
 		else
 			this.unit = unit;
 		units = null;
 	}
 	
 	/** Returns the distance unit (e.g. "micron", "inch"). */
 	public String getUnit() {
 		return unit;
 	}
 	
	/** Returns the plural form of the distance unit (e.g. "microns", "inches"). */
 	public String getUnits() {
 		if (units==null) {
  			if (unit.equals("pixel"))
 				units = "pixels";
 			else if (unit.equals("micron"))
 				units = "microns";
  			else if (unit.equals("inch"))
 				units = "inches";
			else
 				units = unit;
 		}
 		return units;
 	}
 	
   	/** Sets the density unit (e.g. "OD"). */
 	public void setValueUnit(String unit) {
 		if (unit!=null)
 			valueUnit = unit;
 	}
 	
 	/** Returns the density unit. */
 	public String getValueUnit() {
 		return valueUnit;
 	}
 	
 	/** Returns true if this image is density calibrated. */
	public boolean calibrated() {
		return coefficients!=null;
	}
	
	/** Returns the calibration function ID. */
 	public int getFunction() {
 		return function;
 	}
 	
	/** Returns the calibration table. */
 	public float[] getCTable() {
 		return cTable;
 	}
 	
	/** Sets the calibration table. */
 	public void setCTable(float[] table, int function) {
 		cTable = table;
 		this.function = function;
 	}

  	/** Converts a raw pixel value to a density calibrated value. */
 	public double getCValue(int value) {
		//ij.IJ.write("getCValue: "+this);
 		if (cTable!=null && value>=0 && value<cTable.length)
 			return cTable[value];
 		else
 			return value;
 	}
 	
	/** Returns a clone of this object. */
	public Calibration copy() {
		Calibration copy = new Calibration();
		copy.pixelWidth = pixelWidth;
		copy.pixelHeight = pixelHeight;
		copy.pixelDepth = pixelDepth;
		copy.unit = unit;
		copy.units = units;
		copy.valueUnit = valueUnit;
		copy.function = function;
		copy.cTable = cTable;
		return copy;
	}
	
   public String toString() {
    	return
    		"w=" + pixelWidth
			+ ", h=" + pixelHeight
			+ ", d=" + pixelDepth
			+ ", unit=" + unit
 			+ ", cTable=" + (cTable!=null?""+cTable.length:"null");
   }
}


